@@ -5,7 +5,7 @@
 class print_lattice {
 public:
   print_lattice(const Doar::Searcher& sh, const std::string& s,unsigned bg) 
-  : srch(sh),str(s),beg(bg) {}
+    : srch(sh),str(s),beg(bg) {}
   
   void operator()(const char* key, unsigned offset, unsigned id) const {
     printf(" %02X-%02X #%06X: ",beg,beg+offset,id);         // 一致位置とID
@@ -17,10 +17,14 @@ public:
   }
 private:
   const Doar::Searcher& srch;
-  unsigned beg;
   const std::string& str;
+  unsigned beg;
 };
 
+void print_common_prefix(const char* key, unsigned len, unsigned id) {
+  std::cout << " #"<< id <<": "<<std::string(key,key+len)<<" ["<<len<<"]"<< std::endl;	  
+}
+
 
 int main(int argc, char** argv) {
   if(argc != 2) {
@@ -47,19 +51,13 @@ int main(int argc, char** argv) {
     case '+':
       // common prefix search
       {
-	Doar::Node root_node = srch.root_node();
-	unsigned offset=0;
-	std::string key =  word.substr(0,word.size()-1);
-	while((node=srch.common_prefix_search(key.c_str(), offset, root_node)).valid()) {
-	  std::cout << " #"<<node.id()<<": "<<word.substr(0,offset)<<" ["<<offset<<"]"<< std::endl;	  
-	}
+	srch.common_prefix_search(word.c_str(), print_common_prefix);
       }
       break;
     case '~':
       {
 	std::string key =  word.substr(0,word.size()-1);
-	print_lattice pl(srch,key,0);
-	srch.common_prefix_search(key.c_str(), pl);
+	srch.common_prefix_search(key.c_str(), print_lattice(srch,key,0));
       }
       break;
     default:
