@@ -2,12 +2,10 @@
 #define DOAR_NODE_H
 
 namespace Doar{
-  class Builder;
-  class Searcher;
-
   struct Node {
-    Node() : data(0xFFFFFFFF) {}
+    Node() : data(0xFFFFFFFF) {} // TODO: default == root_node にしてもいいかも
     unsigned id()  const { return tail_index(); }
+    //operator bool() const { data!=0xFFFFFFFF; }
     bool valid()   const { return data!=0xFFFFFFFF; }
     bool is_leaf() const { return data&0x80000000; } 
     static const Node INVALID;
@@ -15,6 +13,9 @@ namespace Doar{
   private:
     friend class Builder;
     friend class Searcher;
+    friend class SearcherBase;
+    friend class DoubleArray;
+    friend class DynamicAllocator; // XXX:
 
     explicit Node(unsigned d) : data(d) {}
     
